@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -62,10 +61,13 @@ struct _GthMediaViewerPagePrivate {
 	GdkPixbuf      *icon;
 	PangoLayout    *caption_layout;
 	gboolean        block_next_jump;
+	GdkCursor      *cursor;
+	GdkCursor      *cursor_void;
 };
 
 
 static gpointer gth_media_viewer_page_parent_class = NULL;
+static double default_rates[] = { 0.03, 0.06, 0.12, 0.25, 0.33, 0.50, 0.66, 1.0, 1.50, 2.0, 3.0, 4.0, 8.0, 16.0, 32.0 };
 
 
 static const char *media_viewer_ui_info =
@@ -132,6 +134,10 @@ video_area_realize_cb (GtkWidget *widget,
 {
 	GthMediaViewerPage *self = user_data;
 
+	self->priv->cursor = gdk_cursor_new (GDK_LEFT_PTR);
+	self->priv->cursor_void = gth_cursor_get (gtk_widget_get_window (self->priv->area), GTH_CURSOR_VOID);
+	gdk_window_set_cursor (gtk_widget_get_window (self->priv->area), self->priv->cursor);
+
 	self->priv->caption_layout = gtk_widget_create_pango_layout (widget, "");
 	pango_layout_set_alignment (self->priv->caption_layout, PANGO_ALIGN_CENTER);
 	_gth_media_viewer_page_update_caption (self);
@@ -144,6 +150,16 @@ video_area_unrealize_cb (GtkWidget *widget,
 {
 	GthMediaViewerPage *self = user_data;
 
+	if (self->priv->cursor) {
+		gdk_cursor_unref (self->priv->cursor);
+		self->priv->cursor = NULL;
+	}
+
+	if (self->priv->cursor_void) {
+		gdk_cursor_unref (self->priv->cursor_void);
+		self->priv->cursor_void = NULL;
+	}
+
 	g_object_unref (self->priv->caption_layout);
 	self->priv->caption_layout = NULL;
 }
@@ -345,10 +361,14 @@ position_value_changed_cb (GtkAdjustment *adjustment,
 		return;
 
 	current_value = (gint64) (gtk_adjustment_get_value (adjustment) / 100.0 * self->priv->duration);
-	if (! gst_element_seek_simple (self->priv->playbin,
-				       GST_FORMAT_TIME,
-				       GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT,
-				       current_value))
+	if (! gst_element_seek (self->priv->playbin,
+				self->priv->rate,
+			        GST_FORMAT_TIME,
+			        GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT,
+			        GST_SEEK_TYPE_SET,
+			        current_value,
+			        GST_SEEK_TYPE_NONE,
+			        0.0))
 	{
 		g_warning ("seek failed");
 	}
@@ -400,25 +420,48 @@ hscale_position_button_release_event_cb (GtkWidget      *widget,
 }
 
 
+static void
+update_playback_info (GthMediaViewerPage *self)
+{
+	char *playback_info;
+
+	if (self->priv->playing)
+		playback_info = g_strdup_printf ("@%2.2f", self->priv->rate);
+	else
+		playback_info = g_strdup ("");
+	g_file_info_set_attribute_string (gth_browser_get_current_file (self->priv->browser)->info, "gthumb::statusbar-extra-info", playback_info);
+	gth_browser_update_statusbar_file_info (self->priv->browser);
+
+	g_free (playback_info);
+}
+
+
 static void
 update_player_rate (GthMediaViewerPage *self)
 {
-	self->priv->rate = CLAMP (self->priv->rate, 0.25, 2.0);
+	gint64 current_value;
+
+	self->priv->rate = CLAMP (self->priv->rate,
+				  default_rates[0],
+				  default_rates[G_N_ELEMENTS (default_rates) - 1]);
 
 	if (self->priv->playbin == NULL)
 		return;
 
+	update_playback_info (self);
+
 	if (! self->priv->playing)
 		return;
 
+	current_value = (gint64) (gtk_adjustment_get_value (GTK_ADJUSTMENT (GET_WIDGET ("adjustment_position"))) / 100.0 * self->priv->duration);
 	if (! gst_element_seek (self->priv->playbin,
 				self->priv->rate,
-				GST_FORMAT_TIME,
-				(GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_ACCURATE),
-				GST_SEEK_TYPE_NONE,
-				0.0,
-				GST_SEEK_TYPE_NONE,
-				0.0))
+			        GST_FORMAT_TIME,
+			        GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_ACCURATE,
+			        GST_SEEK_TYPE_SET,
+			        current_value,
+			        GST_SEEK_TYPE_NONE,
+			        0.0))
 	{
 		g_warning ("seek failed");
 	}
@@ -463,13 +506,40 @@ togglebutton_volume_toggled_cb (GtkToggleButton *button,
 }
 
 
+static int
+get_nearest_rate (double rate)
+{
+	int    min_idx = -1;
+	double min_delta;
+	int    i;
+
+	for (i = 0; i < G_N_ELEMENTS (default_rates); i++) {
+		double delta;
+
+		delta = fabs (default_rates[i] - rate);
+		if ((i == 0) || (delta < min_delta)) {
+			min_delta = delta;
+			min_idx = i;
+		}
+	}
+
+	return min_idx;
+}
+
+
 static void
 button_play_slower_clicked_cb (GtkButton *button,
 			       gpointer   user_data)
 {
 	GthMediaViewerPage *self = user_data;
+	int                 i;
+
+	i = get_nearest_rate (self->priv->rate);
+	if (i > 0)
+		self->priv->rate = default_rates[i - 1];
+	else
+		self->priv->rate = default_rates[0];
 
-	self->priv->rate -= 0.25;
 	update_player_rate (self);
 }
 
@@ -479,8 +549,14 @@ button_play_faster_clicked_cb (GtkButton *button,
 			       gpointer   user_data)
 {
 	GthMediaViewerPage *self = user_data;
+	int                 i;
+
+	i = get_nearest_rate (self->priv->rate);
+	if (i < G_N_ELEMENTS (default_rates) - 1)
+		self->priv->rate = default_rates[i + 1];
+	else
+		self->priv->rate = default_rates[G_N_ELEMENTS (default_rates) - 1];
 
-	self->priv->rate += 0.25;
 	update_player_rate (self);
 }
 
@@ -538,6 +614,8 @@ update_play_button (GthMediaViewerPage *self,
 
 		if (self->priv->update_progress_id == 0)
 			self->priv->update_progress_id = gdk_threads_add_timeout (PROGRESS_DELAY, update_progress_cb, self);
+
+		update_playback_info (self);
 	}
 	else if (self->priv->playing && (new_state != GST_STATE_PLAYING)) {
 		self->priv->playing = FALSE;
@@ -548,6 +626,8 @@ update_play_button (GthMediaViewerPage *self,
 			 g_source_remove (self->priv->update_progress_id);
 			 self->priv->update_progress_id = 0;
 		}
+
+		update_playback_info (self);
 	}
 
 	gth_viewer_page_update_sensitivity (GTH_VIEWER_PAGE (self));
@@ -647,6 +727,23 @@ gth_media_viewer_page_real_activate (GthViewerPage *base,
 }
 
 
+static void
+remove_fullscreen_toolbar (GthMediaViewerPage *self)
+{
+	if (self->priv->fullscreen_toolbar == NULL)
+		return;
+
+	g_object_ref (self->priv->mediabar);
+	gtk_container_remove (GTK_CONTAINER (self->priv->fullscreen_toolbar), self->priv->mediabar);
+	gtk_box_pack_start (GTK_BOX (self->priv->area_box), self->priv->mediabar, FALSE, FALSE, 0);
+	g_object_unref (self->priv->mediabar);
+
+	gth_browser_unregister_fullscreen_control (self->priv->browser, self->priv->fullscreen_toolbar);
+	gtk_widget_destroy (self->priv->fullscreen_toolbar);
+	self->priv->fullscreen_toolbar = NULL;
+}
+
+
 static void
 gth_media_viewer_page_real_deactivate (GthViewerPage *base)
 {
@@ -674,6 +771,8 @@ gth_media_viewer_page_real_deactivate (GthViewerPage *base)
 	g_object_unref (self->priv->actions);
 	self->priv->actions = NULL;
 
+	remove_fullscreen_toolbar (self);
+
 	gth_browser_set_viewer_widget (self->priv->browser, NULL);
 }
 
@@ -1005,14 +1104,7 @@ gth_media_viewer_page_real_fullscreen (GthViewerPage *base,
 	GtkAllocation       allocation;
 
 	if (! active) {
-		g_object_ref (self->priv->mediabar);
-		gtk_container_remove (GTK_CONTAINER (self->priv->fullscreen_toolbar), self->priv->mediabar);
-		gtk_box_pack_start (GTK_BOX (self->priv->area_box), self->priv->mediabar, FALSE, FALSE, 0);
-		g_object_unref (self->priv->mediabar);
-
-		gtk_widget_destroy (self->priv->fullscreen_toolbar);
-		self->priv->fullscreen_toolbar = NULL;
-
+		remove_fullscreen_toolbar (self);
 		return;
 	}
 
@@ -1048,11 +1140,13 @@ gth_media_viewer_page_real_show_pointer (GthViewerPage *base,
 	GthMediaViewerPage *self = (GthMediaViewerPage*) base;
 
 	if (self->priv->fullscreen_toolbar != NULL) {
-		if (show && ! gtk_widget_get_visible (self->priv->fullscreen_toolbar)) {
+		if (show) {
 			gtk_widget_show (self->priv->fullscreen_toolbar);
+			gdk_window_set_cursor (gtk_widget_get_window (self->priv->area), self->priv->cursor);
 		}
-		else if (! show && gtk_widget_get_visible (self->priv->fullscreen_toolbar)) {
+		else {
 			gtk_widget_hide (self->priv->fullscreen_toolbar);
+			gdk_window_set_cursor (gtk_widget_get_window (self->priv->area), self->priv->cursor_void);
 		}
 	}
 }
