@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -37,6 +36,9 @@ static const char *fixed_ui_info =
 "    <placeholder name='Tools'>"
 "      <menuitem name='RotateRight' action='Tool_RotateRight'/>"
 "      <menuitem name='RotateLeft' action='Tool_RotateLeft'/>"
+"      <menuitem name='ApplyOrientation' action='Tool_ApplyOrientation'/>"
+"      <menuitem name='ResetOrientation' action='Tool_ResetOrientation'/>"
+"      <separator />"
 "    </placeholder>"
 "  </popup>"
 "</ui>";
@@ -52,6 +54,16 @@ static GtkActionEntry action_entries[] = {
 	  N_("Rotate Left"), "<control><alt>L",
 	  N_("Rotate the selected images 90° to the left"),
 	  G_CALLBACK (gth_browser_activate_action_tool_rotate_left) },
+
+	{ "Tool_ApplyOrientation", NULL,
+	  N_("Rotate Physically"), NULL,
+	  N_("Rotate the selected images according to the embedded orientation"),
+	  G_CALLBACK (gth_browser_activate_action_tool_apply_orientation) },
+
+	{ "Tool_ResetOrientation", NULL,
+	  N_("Reset the EXIF Orientation"), NULL,
+	  N_("Reset the embedded orientation without rotating the images"),
+	  G_CALLBACK (gth_browser_activate_action_tool_reset_orientation) }
 };
 
 
@@ -115,6 +127,12 @@ ir__gth_browser_update_sensitivity_cb (GthBrowser *browser)
 
 	action = gtk_action_group_get_action (data->action_group, "Tool_RotateLeft");
 	g_object_set (action, "sensitive", sensitive, NULL);
+
+	action = gtk_action_group_get_action (data->action_group, "Tool_ApplyOrientation");
+	g_object_set (action, "sensitive", sensitive, NULL);
+
+	action = gtk_action_group_get_action (data->action_group, "Tool_ResetOrientation");
+	g_object_set (action, "sensitive", sensitive, NULL);
 }
 
 
