@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -162,8 +161,6 @@ import_dialog_response_cb (GtkDialog *dialog,
 				GthSubfolderType     subfolder_type;
 				GthSubfolderFormat   subfolder_format;
 				char                *custom_format;
-				gboolean             overwrite_files;
-				gboolean             adjust_orientation;
 				char               **tags;
 				int                  i;
 				GthTask             *task;
@@ -173,8 +170,6 @@ import_dialog_response_cb (GtkDialog *dialog,
 				subfolder_format = eel_gconf_get_enum (PREF_IMPORT_SUBFOLDER_FORMAT, GTH_TYPE_SUBFOLDER_FORMAT, GTH_SUBFOLDER_FORMAT_YYYYMMDD);
 				single_subfolder = eel_gconf_get_boolean (PREF_IMPORT_SUBFOLDER_SINGLE, FALSE);
 				custom_format = eel_gconf_get_string (PREF_IMPORT_SUBFOLDER_CUSTOM_FORMAT, "");
-				overwrite_files = eel_gconf_get_boolean (PREF_IMPORT_OVERWRITE, FALSE);
-				adjust_orientation = eel_gconf_get_boolean (PREF_IMPORT_ADJUST_ORIENTATION, FALSE);
 
 				tags = g_strsplit ((album->keywords != NULL ? album->keywords : ""), ",", -1);
 				for (i = 0; tags[i] != NULL; i++)
@@ -190,8 +185,8 @@ import_dialog_response_cb (GtkDialog *dialog,
 							    (album->title != NULL ? album->title : ""),
 							    tags,
 							    FALSE,
-							    overwrite_files,
-							    adjust_orientation);
+							    FALSE,
+							    FALSE);
 				gth_browser_exec_task (data->browser, task, FALSE);
 				gtk_widget_destroy (data->dialog);
 
@@ -332,8 +327,10 @@ store_password_done_cb (GnomeKeyringResult result,
 
 
 static void account_properties_dialog (DialogData *data,
-			               const char *email);
-static void challange_account_dialog  (DialogData *data);
+			               const char *email,
+			               GError     *error);
+static void challange_account_dialog  (DialogData *data,
+	               	       	       GError     *error);
 
 
 static void
@@ -347,10 +344,12 @@ connection_ready_cb (GObject      *source_object,
 
 	if (! google_connection_connect_finish (conn, result, &error)) {
 		if (g_error_matches (error, GOOGLE_CONNECTION_ERROR, GOOGLE_CONNECTION_ERROR_CAPTCHA_REQUIRED)) {
-			challange_account_dialog (data);
+			challange_account_dialog (data, error);
+			g_clear_error (&error);
 		}
 		else if (g_error_matches (error, GOOGLE_CONNECTION_ERROR, GOOGLE_CONNECTION_ERROR_BAD_AUTHENTICATION)) {
-			account_properties_dialog (data, data->email);
+			account_properties_dialog (data, data->email, error);
+			g_clear_error (&error);
 		}
 		else {
 			if (data->conn != NULL)
@@ -386,6 +385,7 @@ connection_ready_cb (GObject      *source_object,
 
 
 static void connect_to_server (DialogData *data);
+static void auto_select_account (DialogData *data);
 
 
 static void
@@ -417,6 +417,13 @@ account_properties_dialog_response_cb (GtkDialog *dialog,
 		connect_to_server (data);
 		break;
 
+	case PICASA_ACCOUNT_PROPERTIES_RESPONSE_CHOOSE:
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		g_free (data->email);
+		data->email = NULL;
+		auto_select_account (data);
+		break;
+
 	default:
 		break;
 	}
@@ -425,7 +432,8 @@ account_properties_dialog_response_cb (GtkDialog *dialog,
 
 static void
 account_properties_dialog (DialogData *data,
-			   const char *email)
+			   const char *email,
+			   GError     *error)
 {
 	GtkWidget *dialog;
 
@@ -433,6 +441,10 @@ account_properties_dialog (DialogData *data,
 		gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 
 	dialog = picasa_account_properties_dialog_new (email, NULL, NULL);
+	picasa_account_properties_dialog_set_error (PICASA_ACCOUNT_PROPERTIES_DIALOG (dialog), error);
+	if ((error != NULL) && (data->accounts != NULL) && (data->accounts->next != NULL))
+		picasa_account_properties_dialog_can_choose (PICASA_ACCOUNT_PROPERTIES_DIALOG (dialog), TRUE);
+
 	g_signal_connect (dialog,
 			  "response",
 			  G_CALLBACK (account_properties_dialog_response_cb),
@@ -448,9 +460,9 @@ account_properties_dialog (DialogData *data,
 static void
 connect_to_server_step2 (DialogData *data)
 {
-	if (data->password == NULL) {
+	if ((data->password == NULL) || g_str_equal (data->password, "")) {
 		gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
-		account_properties_dialog (data, data->email);
+		account_properties_dialog (data, data->email, NULL);
 	}
 	else {
 		gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
@@ -535,6 +547,13 @@ challange_account_dialog_response_cb (GtkDialog *dialog,
 		connect_to_server (data);
 		break;
 
+	case PICASA_ACCOUNT_PROPERTIES_RESPONSE_CHOOSE:
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		g_free (data->email);
+		data->email = NULL;
+		auto_select_account (data);
+		break;
+
 	default:
 		break;
 	}
@@ -542,11 +561,16 @@ challange_account_dialog_response_cb (GtkDialog *dialog,
 
 
 static void
-challange_account_dialog (DialogData *data)
+challange_account_dialog (DialogData *data,
+			  GError     *error)
 {
 	GtkWidget *dialog;
 
 	dialog = picasa_account_properties_dialog_new (data->email, data->password, google_connection_get_challange_url (data->conn));
+	picasa_account_properties_dialog_set_error (PICASA_ACCOUNT_PROPERTIES_DIALOG (dialog), error);
+	if ((error != NULL) && (data->accounts != NULL) && (data->accounts->next != NULL))
+		picasa_account_properties_dialog_can_choose (PICASA_ACCOUNT_PROPERTIES_DIALOG (dialog), TRUE);
+
 	g_signal_connect (dialog,
 			  "response",
 			  G_CALLBACK (challange_account_dialog_response_cb),
@@ -588,7 +612,7 @@ account_chooser_dialog_response_cb (GtkDialog *dialog,
 
 	case PICASA_ACCOUNT_CHOOSER_RESPONSE_NEW:
 		gtk_widget_destroy (GTK_WIDGET (dialog));
-		account_properties_dialog (data, NULL);
+		account_properties_dialog (data, NULL, NULL);
 		break;
 
 	default:
@@ -613,6 +637,8 @@ auto_select_account (DialogData *data)
 		else {
 			GtkWidget *dialog;
 
+			if (data->conn != NULL)
+				gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
 			dialog = picasa_account_chooser_dialog_new (data->accounts, data->email);
 			g_signal_connect (dialog,
 					  "response",
@@ -626,7 +652,7 @@ auto_select_account (DialogData *data)
 		}
 	}
 	else
-		account_properties_dialog (data, NULL);
+		account_properties_dialog (data, NULL, NULL);
 }
 
 
@@ -701,7 +727,11 @@ account_combobox_changed_cb (GtkComboBox *widget,
 
 	if (g_strcmp0 (email, data->email) != 0) {
 		g_free (data->email);
+		g_free (data->password);
+		g_free (data->challange);
 		data->email = email;
+		data->password = NULL;
+		data->challange = NULL;
 		auto_select_account (data);
 	}
 	else
@@ -800,6 +830,8 @@ album_combobox_changed_cb (GtkComboBox *widget,
 			    ALBUM_DATA_COLUMN, &data->album,
 			    -1);
 
+	gth_import_preferences_dialog_set_event (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog), data->album->title);
+
 	gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
 	picasa_web_service_list_photos (data->picasaweb,
 					data->album,
@@ -896,16 +928,6 @@ file_list_selection_changed_cb (GtkIconView *iconview,
 }
 
 
-static void
-preferences_button_clicked_cb (GtkWidget  *widget,
-			       DialogData *data)
-{
-	gth_import_preferences_dialog_set_event (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog),
-						 (data->album != NULL) ? data->album->title : "");
-	gtk_window_present (GTK_WINDOW (data->preferences_dialog));
-}
-
-
 void
 dlg_import_from_picasaweb (GthBrowser *browser)
 {
@@ -968,6 +990,13 @@ dlg_import_from_picasaweb (GthBrowser *browser)
 	data->preferences_dialog = gth_import_preferences_dialog_new ();
 	gtk_window_set_transient_for (GTK_WINDOW (data->preferences_dialog), GTK_WINDOW (data->dialog));
 
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("destination_button_box")),
+			    gth_import_destination_button_new (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog)),
+			    TRUE,
+			    TRUE,
+			    0);
+	gtk_widget_show_all (GET_WIDGET ("destination_button_box"));
+
 	/* Set the signals handlers. */
 
 	g_signal_connect (G_OBJECT (data->dialog),
@@ -994,12 +1023,9 @@ dlg_import_from_picasaweb (GthBrowser *browser)
 			  "selection_changed",
 			  G_CALLBACK (file_list_selection_changed_cb),
 			  data);
-	g_signal_connect (GET_WIDGET ("preferences_button"),
-			  "clicked",
-			  G_CALLBACK (preferences_button_clicked_cb),
-			  data);
 
 	update_selection_status (data);
+	gth_import_preferences_dialog_set_event (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog), "");
 
 	data->accounts = picasa_web_accounts_load_from_file (&data->email);
 	auto_select_account (data);
