@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -76,6 +75,7 @@ struct _GthSlideshowPrivate {
 #endif
 	gboolean               paused;
 	gboolean               animating;
+	gboolean               random_order;
 };
 
 
@@ -97,6 +97,28 @@ _gth_slideshow_close (GthSlideshow *self)
 }
 
 
+static int
+shuffle_func (gconstpointer a,
+	      gconstpointer b)
+{
+        return g_random_int_range (-1, 2);
+}
+
+
+
+static void
+_gth_slideshow_reset_current (GthSlideshow *self)
+{
+	if (self->priv->random_order)
+		self->priv->file_list = g_list_sort (self->priv->file_list, shuffle_func);
+
+	if (self->priv->direction == GTH_SLIDESHOW_DIRECTION_FORWARD)
+		self->priv->current = g_list_first (self->priv->file_list);
+	else
+		self->priv->current = g_list_last (self->priv->file_list);
+}
+
+
 static void
 _gth_slideshow_load_current_image (GthSlideshow *self)
 {
@@ -114,11 +136,7 @@ _gth_slideshow_load_current_image (GthSlideshow *self)
 			_gth_slideshow_close (self);
 			return;
 		}
-
-		if (self->priv->direction == GTH_SLIDESHOW_DIRECTION_FORWARD)
-			self->priv->current = g_list_first (self->priv->file_list);
-		else
-			self->priv->current = g_list_last (self->priv->file_list);
+		_gth_slideshow_reset_current (self);
 	}
 
 	requested_file = (GthFileData *) self->priv->current->data;
@@ -240,6 +258,7 @@ gth_slideshow_init (GthSlideshow *self)
 	self->priv->paused = FALSE;
 	self->priv->animating = FALSE;
 	self->priv->direction = GTH_SLIDESHOW_DIRECTION_FORWARD;
+	self->priv->random_order = FALSE;
 
 	self->priv->preloader = gth_image_preloader_new ();
 	g_signal_connect (self->priv->preloader,
@@ -384,28 +403,31 @@ gth_slideshow_show_cb (GtkWidget    *widget,
 	if (! self->priv->first_show)
 		return;
 
-#if HAVE_GSTREAMER
-	if (gstreamer_init ()) {
-		if ((self->priv->audio_files != NULL) && (self->priv->audio_files[0] != NULL)) {
-			self->priv->current_audio_file = 0;
-			if (self->priv->playbin == NULL) {
-				GstBus *bus;
+	self->priv->first_show = FALSE;
 
-				self->priv->playbin = gst_element_factory_make ("playbin", "playbin");
-				bus = gst_pipeline_get_bus (GST_PIPELINE (self->priv->playbin));
-				gst_bus_add_signal_watch (bus);
-				g_signal_connect (bus, "message", G_CALLBACK (bus_message_cb), self);
-			}
-			else
-				gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
-			g_object_set (G_OBJECT (self->priv->playbin), "uri", self->priv->audio_files[self->priv->current_audio_file], NULL);
-			gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
+#if HAVE_GSTREAMER
+	if ((self->priv->audio_files != NULL)
+	    && (self->priv->audio_files[0] != NULL)
+	    && gstreamer_init ())
+	{
+		self->priv->current_audio_file = 0;
+		if (self->priv->playbin == NULL) {
+			GstBus *bus;
+
+			self->priv->playbin = gst_element_factory_make ("playbin", "playbin");
+			bus = gst_pipeline_get_bus (GST_PIPELINE (self->priv->playbin));
+			gst_bus_add_signal_watch (bus);
+			g_signal_connect (bus, "message", G_CALLBACK (bus_message_cb), self);
 		}
+		else
+			gst_element_set_state (self->priv->playbin, GST_STATE_NULL);
+		g_object_set (G_OBJECT (self->priv->playbin), "uri", self->priv->audio_files[self->priv->current_audio_file], NULL);
+		gst_element_set_state (self->priv->playbin, GST_STATE_PLAYING);
 	}
 #endif
 
+	_gth_slideshow_reset_current (self);
 	_gth_slideshow_load_current_image (self);
-	self->priv->first_show = FALSE;
 }
 
 
@@ -418,7 +440,6 @@ _gth_slideshow_construct (GthSlideshow *self,
 	self->priv->projector = projector;
 	self->priv->browser = _g_object_ref (browser);
 	self->priv->file_list = _g_object_list_ref (file_list);
-	self->priv->current = self->priv->file_list;
 	self->priv->one_loaded = FALSE;
 
 	self->priv->projector->construct (self);
@@ -486,6 +507,14 @@ gth_slideshow_set_playlist (GthSlideshow  *self,
 }
 
 
+void
+gth_slideshow_set_random_order (GthSlideshow *self,
+				gboolean      random)
+{
+	self->priv->random_order = random;
+}
+
+
 /* -- default projector -- */
 
 
