@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef GTH_BROWSER_H
@@ -106,6 +105,7 @@ GtkWidget *      gth_browser_get_infobar            (GthBrowser       *browser);
 GtkWidget *      gth_browser_get_statusbar          (GthBrowser       *browser);
 GtkWidget *      gth_browser_get_file_list          (GthBrowser       *browser);
 GtkWidget *      gth_browser_get_file_list_view     (GthBrowser       *browser);
+GtkWidget *      gth_browser_get_thumbnail_list     (GthBrowser       *browser);
 GtkWidget *      gth_browser_get_thumbnail_list_view(GthBrowser       *browser);
 GthFileSource *  gth_browser_get_location_source    (GthBrowser       *browser);
 GthFileStore *   gth_browser_get_file_store         (GthBrowser       *browser);
@@ -152,6 +152,8 @@ void             gth_browser_load_file              (GthBrowser       *browser,
 void             gth_browser_update_title           (GthBrowser       *browser);
 void             gth_browser_update_sensitivity     (GthBrowser       *browser);
 void             gth_browser_update_extra_widget    (GthBrowser       *browser);
+void             gth_browser_update_statusbar_file_info
+						    (GthBrowser       *browser);
 void             gth_browser_show_viewer_properties (GthBrowser       *browser,
 						     gboolean          show);
 void             gth_browser_show_viewer_tools      (GthBrowser       *browser,
@@ -166,6 +168,9 @@ gpointer         gth_browser_get_image_preloader    (GthBrowser       *browser);
 void             gth_browser_register_fullscreen_control
 						    (GthBrowser       *browser,
 					             GtkWidget        *widget);
+void             gth_browser_unregister_fullscreen_control
+						    (GthBrowser       *browser,
+					             GtkWidget        *widget);
 void             gth_browser_fullscreen             (GthBrowser       *browser);
 void             gth_browser_unfullscreen           (GthBrowser       *browser);
 void             gth_browser_file_menu_popup        (GthBrowser       *browser,
