@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -135,7 +134,7 @@ gth_file_properties_real_set_file (GthPropertyView *base,
 			g_free (value);
 			value = tmp_value;
 		}
-		tooltip = g_markup_printf_escaped ("%s: %s", info->display_name, value);
+		tooltip = g_markup_printf_escaped ("%s: %s", _(info->display_name), value);
 
 		category = g_hash_table_lookup (category_hash, info->category);
 		if (category == NULL) {
@@ -155,7 +154,7 @@ gth_file_properties_real_set_file (GthPropertyView *base,
 				    &iter,
 				    WEIGHT_COLUMN, PANGO_WEIGHT_NORMAL,
 				    ID_COLUMN, info->id,
-				    DISPLAY_NAME_COLUMN, info->display_name,
+				    DISPLAY_NAME_COLUMN, _(info->display_name),
 				    VALUE_COLUMN, value,
 				    TOOLTIP_COLUMN, tooltip,
 				    POS_COLUMN, (category->sort_order * CATEGORY_SIZE) + info->sort_order,
